@@ -89,13 +89,13 @@ protected:
       
   FoundSet m_found_set;
 
-  type_vecFields m_TableFields; //A cache, so we don't have to repeatedly get them from the Document.
+  type_vec_fields m_TableFields; //A cache, so we don't have to repeatedly get them from the Document.
   type_vecLayoutFields m_FieldsShown; //And any extra keys needed by shown fields.
 
   type_signal_record_changed m_signal_record_changed;
     
 private:
-  bool get_related_record_exists(const sharedptr<const Relationship>& relationship, const sharedptr<const Field>& key_field, const Gnome::Gda::Value& key_value);
+  bool get_related_record_exists(const sharedptr<const Relationship>& relationship, const Gnome::Gda::Value& key_value);
 };
 
 } //namespace Glom
