@@ -24,7 +24,10 @@
 #include <libglom/document/document.h>
 #include <libglom/utils.h>
 //#include <libgdamm/connectionevent.h>
-#include <glibmm/i18n.h>
+
+#include <libglom/connectionpool_backends/postgres_central.h>
+#include <libglom/connectionpool_backends/postgres_self.h>
+# include <libglom/connectionpool_backends/sqlite.h>
 
 #ifdef G_OS_WIN32
 # include <windows.h>
@@ -35,6 +38,9 @@
 
 #include <signal.h> //To catch segfaults
 
+#include <glibmm/i18n.h>
+
+
 #ifndef G_OS_WIN32
 static EpcProtocol publish_protocol = EPC_PROTOCOL_HTTPS;
 #endif
@@ -97,7 +103,8 @@ ConnectionPool::ConnectionPool()
   m_backend(0),
   m_sharedconnection_refcount(0),
   m_ready_to_connect(false),
-  m_pFieldTypes(0)
+  m_pFieldTypes(0),
+  m_show_debug_output(false)
 {
 }
 
@@ -123,6 +130,48 @@ ConnectionPool* ConnectionPool::get_instance()
   }
 }
 
+void ConnectionPool::setup_from_document(const Document* document)
+{
+  switch(document->get_hosting_mode())
+  {
+  case Document::HOSTING_MODE_POSTGRES_SELF:
+    {
+      ConnectionPoolBackends::PostgresSelfHosted* backend = new ConnectionPoolBackends::PostgresSelfHosted;
+      backend->set_self_hosting_data_uri(document->get_connection_self_hosted_directory_uri());
+      set_backend(std::auto_ptr<ConnectionPool::Backend>(backend));
+    }
+    break;
+  case Document::HOSTING_MODE_POSTGRES_CENTRAL:
+    {
+      ConnectionPoolBackends::PostgresCentralHosted* backend = new ConnectionPoolBackends::PostgresCentralHosted;
+      backend->set_host(document->get_connection_server());
+      backend->set_port(document->get_connection_port());
+      backend->set_try_other_ports(document->get_connection_try_other_ports());
+      set_backend(std::auto_ptr<ConnectionPool::Backend>(backend));
+    }
+    break;
+  case Document::HOSTING_MODE_SQLITE:
+    {
+      ConnectionPoolBackends::Sqlite* backend = new ConnectionPoolBackends::Sqlite;
+      backend->set_database_directory_uri(document->get_connection_self_hosted_directory_uri());
+      set_backend(std::auto_ptr<ConnectionPool::Backend>(backend));
+    }
+    break;
+
+  default:
+    //on_document_load() should have checked for this already, informing the user.
+    std::cerr << "Glom: setup_connection_pool_from_document(): Unhandled hosting mode: " << document->get_hosting_mode() << std::endl;
+    g_assert_not_reached();
+    break;
+  }
+
+  // Might be overwritten later when actually attempting a connection:
+  set_user(document->get_connection_user());
+  set_database(document->get_connection_database());
+
+  set_ready_to_connect();
+}
+  
 void ConnectionPool::delete_instance()
 {
   if(m_instance)
@@ -540,7 +589,7 @@ bool ConnectionPool::startup(const SlotProgress& slot_progress, bool network_sha
   return true;
 }
 
-void ConnectionPool::cleanup(const SlotProgress& slot_progress)
+bool ConnectionPool::cleanup(const SlotProgress& slot_progress)
 {
   // Without a valid backend instance we should not state that we are ready to
   // connect. Fixes crash described in #577821.
@@ -551,8 +600,10 @@ void ConnectionPool::cleanup(const SlotProgress& slot_progress)
   // set_not_ready_to_connect()?
   set_ready_to_connect(false);
 
+  bool result = false;
+  
   if(m_backend.get())
-    m_backend->cleanup(slot_progress);
+    result = m_backend->cleanup(slot_progress);
 
   //Make sure that connect() tries to make a new connection:
   invalidate_connection();
@@ -566,6 +617,8 @@ void ConnectionPool::cleanup(const SlotProgress& slot_progress)
   //We don't need the segfault handler anymore:
   signal(SIGSEGV, previous_sig_handler);
   previous_sig_handler = SIG_DFL; /* Arbitrary default */
+  
+  return result;
 }
 
 
@@ -896,5 +949,14 @@ void ConnectionPool::set_get_document_func(const SlotGetDocument& slot)
   m_slot_get_document = slot;
 }
 
+void ConnectionPool::set_show_debug_output(bool val)
+{
+  m_show_debug_output = val;
+}
+ 
+bool ConnectionPool::get_show_debug_output() const
+{
+  return m_show_debug_output;
+}
 
 } //namespace Glom
