@@ -18,9 +18,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include "layoutgroup.h"
-#include "layoutitem_field.h"
-#include "layoutitem_portal.h"
+#include <libglom/data_structure/layout/layoutgroup.h>
+#include <libglom/data_structure/layout/layoutitem_field.h>
+#include <libglom/data_structure/layout/layoutitem_portal.h>
 #include <glibmm/i18n.h>
 
 namespace Glom
@@ -415,7 +415,16 @@ void LayoutGroup::debug(guint level) const
 }
 */
 
+guint LayoutGroup::get_columns_count() const
+{
+  return m_columns_count;
+}
 
+void LayoutGroup::set_columns_count(guint columns_count)
+{
+  m_columns_count = columns_count;
+}
+  
 double LayoutGroup::get_border_width() const
 {
   return m_border_width;
