@@ -22,24 +22,31 @@
 #define GLOM_DIALOG_DESIGN_H
 
 #include <gtkmm.h>
-#include <libglademm.h>
-#include <glom/libglom/document/document_glom.h>
+#include <gtkmm/builder.h>
+#include <libglom/document/view.h>
 
+namespace Glom
+{
+
+/** A dialog with a titled frame, a label for the table title, and a close button.
+ */
 class Dialog_Design
-: public Gtk::Dialog,
+: public Gtk::Window,
   public View_Composite_Glom
 {
 public:
-  Dialog_Design(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
+  Dialog_Design(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder);
   virtual ~Dialog_Design();
 
   virtual bool init_db_details(const Glib::ustring& table_name);
 
-protected:
+private:
   virtual void on_button_close();
 
+protected:
   Gtk::Label* m_label_table;
-  Gtk::Label* m_label_frame;
 };
 
+} //namespace Glom
+
 #endif //GLOM_DIALOG_DESIGN_H
