@@ -23,6 +23,7 @@
 #include <glom/glade_utils.h>
 #include <glom/utils_ui.h> //For bold_message()).
 #include <libglom/libglom_config.h>
+#include <libglom/db_utils.h>
 #include <glibmm/i18n.h>
 
 namespace Glom
@@ -43,9 +44,7 @@ void Box_DB_Table_Definition::init()
 {
   //m_strHint = _("Click [Edit] to edit the field definition in more detail.\nUse the Mode menu to see Data or perform a Find.");
 
-  Glib::RefPtr<Gtk::Builder> refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path("glom_developer.glade"), "window_field_definition_edit");
-  if(refXml)
-    refXml->get_widget_derived("window_field_definition_edit", m_pDialog);
+  Utils::get_glade_widget_derived_with_warning(m_pDialog);
 
   add_view(m_pDialog); //Give it access to the document.
 
@@ -179,7 +178,7 @@ void Box_DB_Table_Definition::on_adddel_add(const Gtk::TreeModel::iterator& row)
 
     //TODO: Warn about a delay before actually doing this when the backend
     //needs to recreate the whole table.
-    const bool bTest = add_column(m_table_name, field, get_app_window()); //TODO: Get schema type for Field::TYPE_NUMERIC
+    const bool bTest = DbUtils::add_column(m_table_name, field, get_app_window()); //TODO: Get schema type for Field::TYPE_NUMERIC
     if(bTest)
     {
       //Store the generated title in the document:
@@ -231,7 +230,7 @@ void Box_DB_Table_Definition::on_adddel_delete(const Gtk::TreeModel::iterator& r
     {
       //TODO: Warn about a delay before actually doing this when the backend
       //needs to recreate the whole table.
-      const bool test = drop_column(m_table_name, name, get_app_window());
+      const bool test = DbUtils::drop_column(m_table_name, name);
       if(test)
       {
         //update_gda_metastore_for_table(m_table_name); // already done in drop_column().
@@ -324,7 +323,7 @@ bool Box_DB_Table_Definition::check_field_change(const sharedptr<const Field>& f
 
   //Refuse to change a field name to the same as an existing one:
   if( (field_new->get_name() != field_old->get_name()) &&
-      (get_field_exists_in_database(m_table_name, field_new->get_name())) )
+      (DbUtils::get_field_exists_in_database(m_table_name, field_new->get_name())) )
   {
     std::cout << "get_field_exists_in_database(" << m_table_name << ", " << field_new->get_name() << ") returned true" << std::endl;
 
@@ -623,14 +622,18 @@ void Box_DB_Table_Definition::fill_fields()
 
 bool Box_DB_Table_Definition::field_has_null_values(const sharedptr<const Field>& field)
 {
-  //TODO: Use SQL paramters?
   //Note that "= Null" doesn't work, though it doesn't error either.
   //Note also that SELECT COUNT always returns 0 if all the values are NULL, so we can't use that to be more efficient.
-  const Glib::ustring sql_query = "SELECT \"" + field->get_name() + "\" FROM \"" + m_table_name + "\" WHERE \"" + m_table_name + "\".\"" + field->get_name() + "\" IS NULL LIMIT 1";
-  //std::cout << "sql_query: " << sql_query << std::endl;
+  Glib::RefPtr<Gnome::Gda::SqlBuilder> builder =
+    Gnome::Gda::SqlBuilder::create(Gnome::Gda::SQL_STATEMENT_SELECT);
+  builder->select_add_field(field->get_name(), m_table_name);
+  builder->select_add_target(m_table_name);
+  builder->set_where(
+    builder->add_cond(Gnome::Gda::SQL_OPERATOR_TYPE_ISNULL,
+      builder->add_id(field->get_name()))); //TODO: It would nice to specify the table name here too.
 
   long null_count = 0;
-  Glib::RefPtr<Gnome::Gda::DataModel> datamodel = query_execute_select(sql_query);
+  Glib::RefPtr<Gnome::Gda::DataModel> datamodel = DbUtils::query_execute_select(builder);
   if(datamodel)
   {
     if(datamodel->get_n_rows() && datamodel->get_n_columns())
@@ -641,7 +644,7 @@ bool Box_DB_Table_Definition::field_has_null_values(const sharedptr<const Field>
   }
   else
   {
-    std::cerr << "Box_DB_Table_Definition::field_has_null_values(): query failed: " << sql_query << std::endl;
+    std::cerr << "Box_DB_Table_Definition::field_has_null_values(): query failed." << std::endl;
   }
 
   return null_count > 0; 
@@ -652,10 +655,13 @@ bool Box_DB_Table_Definition::field_has_non_unique_values(const sharedptr<const
   long count_distinct = 0;
   long count_all = 0;
 
-  //Count the distint rows:
-  const Glib::ustring sql_query_distinct = "SELECT DISTINCT \"" + field->get_name() + "\" FROM \"" + m_table_name + "\"";
-  
-  Glib::RefPtr<Gnome::Gda::DataModel> datamodel = query_execute_select(sql_query_distinct);
+  //Count the distinct rows:  
+  Glib::RefPtr<Gnome::Gda::SqlBuilder> builder_query_distinct = Gnome::Gda::SqlBuilder::create(Gnome::Gda::SQL_STATEMENT_UPDATE);
+  builder_query_distinct->select_set_distinct();
+  builder_query_distinct->set_table(m_table_name);
+  builder_query_distinct->select_add_field(field->get_name(), m_table_name);
+
+  Glib::RefPtr<Gnome::Gda::DataModel> datamodel = DbUtils::query_execute_select(builder_query_distinct);
   if(datamodel)
   {
     count_distinct = datamodel->get_n_rows();
@@ -667,9 +673,12 @@ bool Box_DB_Table_Definition::field_has_non_unique_values(const sharedptr<const
   }
 
   //Count all rows, to compare. TODO_performance: Is there a more efficient way to do this? Maybe count(*), which apparently doesn't ignore NULL rows like count(somefield) would.
-  const Glib::ustring sql_query_all = "SELECT \"" + field->get_name() + "\" FROM \"" + m_table_name + "\"";
-  
-  datamodel = query_execute_select(sql_query_all);
+  Glib::RefPtr<Gnome::Gda::SqlBuilder> builder_query_all =
+    Gnome::Gda::SqlBuilder::create(Gnome::Gda::SQL_STATEMENT_SELECT);
+  builder_query_all->select_add_field(field->get_name(), m_table_name);
+  builder_query_all->select_add_target(m_table_name);
+      
+  datamodel = DbUtils::query_execute_select(builder_query_all);
   if(datamodel)
   {
     count_all = datamodel->get_n_rows();
