@@ -21,220 +21,16 @@
 #include <iostream>
 #include "glom_db_treemodel.h"
 
-#include <glom/libglom/connectionpool.h>
-#include <glom/libglom/data_structure/glomconversions.h> //For util_build_sql
-#include <glom/libglom/utils.h>
+#include <libglom/connectionpool.h>
+#include <libglom/data_structure/glomconversions.h> //For util_build_sql
+#include <libglom/utils.h>
+#include <libglom/db_utils.h>
 
 #include "glom/application.h"
 
 namespace Glom
 {
 
-#ifndef GLIBMM_VFUNCS_ENABLED
-  // C vfuncs implementation. Merely copied from gtkmm.
-  GtkTreeModelFlags DbTreeModel::glom_get_flags_impl(GtkTreeModel* model)
-  {
-    Glib::ObjectBase* const obj_base = static_cast<Glib::ObjectBase*>(Glib::ObjectBase::_get_current_wrapper((GObject*)model));
-    if(obj_base)
-    {
-      Glom::DbTreeModel* cpp_model = dynamic_cast<Glom::DbTreeModel*>(obj_base);
-      g_assert(cpp_model);
-      return GtkTreeModelFlags(cpp_model->get_flags_vfunc());
-    }
-    else
-      return GtkTreeModelFlags(0);
-  }
-
-  gint DbTreeModel::glom_get_n_columns_impl(GtkTreeModel* model)
-  {
-    Glib::ObjectBase* const obj_base = static_cast<Glib::ObjectBase*>(Glib::ObjectBase::_get_current_wrapper((GObject*)model));
-    if(obj_base)
-    {
-      Glom::DbTreeModel* cpp_model = dynamic_cast<Glom::DbTreeModel*>(obj_base);
-      g_assert(cpp_model);
-      return cpp_model->get_n_columns_vfunc();
-    }
-    else
-      return 0;
-  }
-  
-  GType DbTreeModel::glom_get_column_type_impl(GtkTreeModel* model, gint index)
-  {
-    Glib::ObjectBase* const obj_base = static_cast<Glib::ObjectBase*>(Glib::ObjectBase::_get_current_wrapper((GObject*)model));
-    if(obj_base)
-    {
-      Glom::DbTreeModel* cpp_model = dynamic_cast<Glom::DbTreeModel*>(obj_base);
-      g_assert(cpp_model);
-      return cpp_model->get_column_type_vfunc(index);
-    }
-    else
-      return G_TYPE_NONE;
-  }
-
-  void DbTreeModel::glom_get_value_impl(GtkTreeModel* model, GtkTreeIter* iter, gint column, GValue* value)
-  {
-    Glib::ObjectBase* const obj_base = static_cast<Glib::ObjectBase*>(Glib::ObjectBase::_get_current_wrapper((GObject*)model));
-    if(obj_base)
-    {
-      Glom::DbTreeModel* cpp_model = dynamic_cast<Glom::DbTreeModel*>(obj_base);
-      g_assert(cpp_model);
-      cpp_model->get_value_vfunc(Gtk::TreeModel::iterator(model, iter), column, *reinterpret_cast<Glib::ValueBase*>(value));
-    }
-  }
-
-  gboolean DbTreeModel::glom_iter_next_impl(GtkTreeModel* model, GtkTreeIter* iter)
-  {
-    Glib::ObjectBase* const obj_base = static_cast<Glib::ObjectBase*>(Glib::ObjectBase::_get_current_wrapper((GObject*)model));
-    if(obj_base)
-    {
-      Glom::DbTreeModel* cpp_model = dynamic_cast<Glom::DbTreeModel*>(obj_base);
-      g_assert(cpp_model);
-
-      TreeModel::iterator iter_input = TreeModel::iterator(model, iter);
-      TreeModel::iterator iter_next(model, iter);
-      const gboolean retval = cpp_model->iter_next_vfunc(iter_input, iter_next);
-
-      if(retval)
-        *iter = *(iter_next.gobj());
-      
-      return retval;
-    }
-    else
-      return FALSE;
-  }
-
-  gboolean DbTreeModel::glom_iter_children_impl(GtkTreeModel* model, GtkTreeIter* iter, GtkTreeIter* parent)
-  {
-    Glib::ObjectBase* const obj_base = static_cast<Glib::ObjectBase*>(Glib::ObjectBase::_get_current_wrapper((GObject*)model));
-    if(obj_base)
-    {
-      Glom::DbTreeModel* cpp_model = dynamic_cast<Glom::DbTreeModel*>(obj_base);
-      g_assert(cpp_model);
-
-      TreeModel::iterator iter_out(model, iter);
-      const gboolean retval = cpp_model->iter_children_vfunc(TreeModel::iterator(model, parent), iter_out);
-
-      if(retval)
-        *iter = *(iter_out.gobj());
-      
-      return retval;
-    }
-    else
-      return FALSE;
-  }
-
-  gboolean DbTreeModel::glom_iter_has_child_impl(GtkTreeModel* model, GtkTreeIter* iter)
-  {
-    Glib::ObjectBase* const obj_base = static_cast<Glib::ObjectBase*>(Glib::ObjectBase::_get_current_wrapper((GObject*)model));
-    if(obj_base)
-    {
-      Glom::DbTreeModel* cpp_model = dynamic_cast<Glom::DbTreeModel*>(obj_base);
-      g_assert(cpp_model);
-      return cpp_model->iter_has_child_vfunc(TreeModel::iterator(model, iter));
-    }
-    else
-      return FALSE;
-  }
-
-  gint DbTreeModel::glom_iter_n_children_impl(GtkTreeModel* model, GtkTreeIter* iter)
-  {
-    Glib::ObjectBase* const obj_base = static_cast<Glib::ObjectBase*>(Glib::ObjectBase::_get_current_wrapper((GObject*)model));
-    if(obj_base)
-    {
-      Glom::DbTreeModel* cpp_model = dynamic_cast<Glom::DbTreeModel*>(obj_base);
-      g_assert(cpp_model);
-      return cpp_model->iter_n_children_vfunc(TreeModel::iterator(model, iter));
-    }
-    else
-      return 0;
-  }
-
-  gboolean DbTreeModel::glom_iter_nth_child_impl(GtkTreeModel* model, GtkTreeIter* iter, GtkTreeIter* parent, gint n)
-  {
-    Glib::ObjectBase* const obj_base = static_cast<Glib::ObjectBase*>(Glib::ObjectBase::_get_current_wrapper((GObject*)model));
-    if(obj_base)
-    {
-      Glom::DbTreeModel* cpp_model = dynamic_cast<Glom::DbTreeModel*>(obj_base);
-      g_assert(cpp_model);
-
-      TreeModel::iterator iter_out(model, iter);
-      gboolean retval = false;
-
-      // Deal with this special case, documented in the C docs as:
-      // "As a special case, if @parent is %NULL, then the nth root node is set.":
-      if(!parent)
-        retval = cpp_model->iter_nth_root_child_vfunc(n, iter_out);
-      else
-        retval = cpp_model->iter_nth_child_vfunc(TreeModel::iterator(model, parent), n, iter_out);
-
-      if(retval)
-        *iter = *(iter_out.gobj());
-
-      return retval;
-    }
-    else
-      return FALSE;
-  }
-
-  gboolean DbTreeModel::glom_iter_parent_impl(GtkTreeModel* model, GtkTreeIter* iter, GtkTreeIter* child)
-  {
-    Glib::ObjectBase* const obj_base = static_cast<Glib::ObjectBase*>(Glib::ObjectBase::_get_current_wrapper((GObject*)model));
-    if(obj_base)
-    {
-      Glom::DbTreeModel* cpp_model = dynamic_cast<Glom::DbTreeModel*>(obj_base);
-      g_assert(cpp_model);
-
-      TreeModel::iterator iter_out(model, iter);
-      const gboolean retval = cpp_model->iter_parent_vfunc(TreeModel::iterator(model, child), iter_out);
-
-      if(retval)
-        *iter = *(iter_out.gobj());
-
-      return retval;
-    }
-    else
-      return FALSE;
-  }
-
-  GtkTreePath* DbTreeModel::glom_get_path_impl(GtkTreeModel* model, GtkTreeIter* iter)
-  {
-    Glib::ObjectBase* const obj_base = static_cast<Glib::ObjectBase*>(Glib::ObjectBase::_get_current_wrapper((GObject*)model));
-    if(obj_base)
-    {
-      Glom::DbTreeModel* cpp_model = dynamic_cast<Glom::DbTreeModel*>(obj_base);
-      g_assert(cpp_model);
-      return cpp_model->get_path_vfunc(TreeModel::iterator(model, iter)).gobj_copy();
-    }
-    else
-      return NULL;
-  }
-
-  gboolean DbTreeModel::glom_get_iter_impl(GtkTreeModel* model, GtkTreeIter* iter, GtkTreePath* path)
-  {
-    Glib::ObjectBase* const obj_base = static_cast<Glib::ObjectBase*>(Glib::ObjectBase::_get_current_wrapper((GObject*)model));
-    if(obj_base)
-    {
-      Glom::DbTreeModel* cpp_model = dynamic_cast<Glom::DbTreeModel*>(obj_base);
-      g_assert(cpp_model);
-
-      TreeModel::iterator iter_out(model, iter);
-
-      const gboolean retval = cpp_model->get_iter_vfunc(Gtk::TreeModel::Path(path, true), iter_out);
-
-      if(retval)
-        *iter = *(iter_out.gobj());
-
-      return retval;
-    }
-    else
-      return FALSE;
-  }
-#endif // !GLIBMM_VFUNCS_ENABLED
-} // namespace Glom
-
-namespace Glom
-{
-
 DbTreeModelRow::DbTreeModelRow()
 : m_values_retrieved(false),
   m_removed(false),
@@ -249,7 +45,7 @@ void DbTreeModelRow::fill_values_if_necessary(DbTreeModel& model, int row)
   //std::cout << "DbTreeModelRow::fill_values_if_necessary(): row=" << row << std::endl;
   //if(row == 1000)
   //{
-  //  std::cout << "1000" << std::endl;  
+  //  std::cout << "1000" << std::endl;
   //}
 
   if(m_values_retrieved)
@@ -259,7 +55,7 @@ void DbTreeModelRow::fill_values_if_necessary(DbTreeModel& model, int row)
   else
   {
     //std::cout << "debug: DbTreeModelRow::fill_values_if_necessary(): retrieving for row=" << row << std::endl;
-  
+
     if((row < (int)model.m_data_model_rows_count) && model.m_gda_datamodel)
     {
       Glib::RefPtr<Gnome::Gda::DataModelIter> iter = model.m_gda_datamodel->create_iter();
@@ -276,7 +72,7 @@ void DbTreeModelRow::fill_values_if_necessary(DbTreeModel& model, int row)
             m_db_values[i] = holder->get_value(); //TODO_gda: Why not just use get_value_at()?
           else
           {
-            // This is quite possible, for example for unset dates. jhs 
+            // This is quite possible, for example for unset dates. jhs
             //std::cerr << "DbTreeModelRow::fill_values_if_necessary(): NULL Gnome::Gda::Holder for field=" << i << std::endl;
           }
 
@@ -299,7 +95,7 @@ void DbTreeModelRow::fill_values_if_necessary(DbTreeModel& model, int row)
       if(m_extra)
       {
         //std::cout << "  debug: DbTreeModelRow::fill_values_if_necessary(): using default value" << std::endl;
-  
+
         //It is an extra row, added with append().
       }
       else if(!m_removed)
@@ -316,7 +112,7 @@ void DbTreeModelRow::fill_values_if_necessary(DbTreeModel& model, int row)
         {
           Glib::RefPtr<Gnome::Gda::Column> column = model.m_gda_datamodel->describe_column(col);
 
-          //We don't just create a Gda::Value of the column's gda type, 
+          //We don't just create a Gda::Value of the column's gda type,
           //because we should use a NULL-type Gda::Value as the initial value for some fields:
           const Field::glom_field_type glom_type = Field::get_glom_type_for_gda_type(column->get_g_type());
           m_db_values[col] = Glom::Conversions::get_empty_value(glom_type);
@@ -390,22 +186,6 @@ DbTreeModel::DbTreeModel(const Gtk::TreeModelColumnRecord& columns, const FoundS
     //GType gtype = G_OBJECT_TYPE(gobj());  //The custom GType created in the Object constructor, from the typeid.
     //Gtk::TreeModel::add_interface( gtype );
 
-#ifndef GLIBMM_VFUNCS_ENABLED
-    GtkTreeModelIface* iface = GTK_TREE_MODEL_GET_IFACE(gobj());
-    iface->get_flags = glom_get_flags_impl;
-    iface->get_n_columns = glom_get_n_columns_impl;
-    iface->get_column_type = glom_get_column_type_impl;
-    iface->get_value = glom_get_value_impl;
-    iface->iter_next = glom_iter_next_impl;
-    iface->iter_children = glom_iter_children_impl;
-    iface->iter_has_child = glom_iter_has_child_impl;
-    iface->iter_n_children = glom_iter_n_children_impl;
-    iface->iter_nth_child = glom_iter_nth_child_impl;
-    iface->iter_parent = glom_iter_parent_impl;
-    iface->get_path = glom_get_path_impl;
-    iface->get_iter = glom_get_iter_impl;
-#endif // !GLIBMM_VFUNCS_ENABLED
-
     m_iface_initialized = true; //Prevent us from calling add_interface() on the same gtype again.
   }
 
@@ -443,7 +223,7 @@ bool DbTreeModel::refresh_from_database(const FoundSet& found_set)
   {
     m_get_records = false; //Otherwise it would not make sense.
 
-    //Use a dummy DataModel that has the same columns and types, 
+    //Use a dummy DataModel that has the same columns and types,
     //but which does not use a real database table,
     //so we can use it to add find criteria.
     Glib::RefPtr<Gnome::Gda::DataModelArray> model_array = Gnome::Gda::DataModelArray::create(m_column_fields.size());
@@ -463,7 +243,12 @@ bool DbTreeModel::refresh_from_database(const FoundSet& found_set)
     }
 
     //Add at least an initial row:
-    m_gda_datamodel->append_row(); //TODO: Handle adding.
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+  m_gda_datamodel->append_row(); //TODO: Handle adding.
+#else
+  std::auto_ptr<Glib::Error> error;
+  m_gda_datamodel->append_row(error); //TODO: Handle adding.
+#endif
     return true;
   }
 
@@ -483,37 +268,25 @@ bool DbTreeModel::refresh_from_database(const FoundSet& found_set)
 
   if(m_found_set.m_table_name.empty())
     std::cerr << "DEBUG: refresh_from_database(): found_set.m_table_name is empty." << std::endl;
-    
+
   if(m_connection && !m_found_set.m_table_name.empty() && m_get_records)
   {
-    const Glib::ustring sql_query = Utils::build_sql_select_with_where_clause(m_found_set.m_table_name, m_column_fields, m_found_set.m_where_clause, m_found_set.m_extra_join, m_found_set.m_sort_clause, m_found_set.m_extra_group_by);
+    Glib::RefPtr<Gnome::Gda::SqlBuilder> sql_query = Utils::build_sql_select_with_where_clause(m_found_set.m_table_name, m_column_fields, m_found_set.m_where_clause, m_found_set.m_extra_join, m_found_set.m_sort_clause);
     //std::cout << "  Debug: DbTreeModel::refresh_from_database():  " << sql_query << std::endl;
 
-    const App_Glom* app = App_Glom::get_application();
-    if(app && app->get_show_sql_debug())
+    m_gda_datamodel = DbUtils::query_execute_select(sql_query, true /* use_cursor */);
+
+    if(!m_gda_datamodel)
     {
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-      try
-      {
-#endif //GLIBMM_EXCEPTIONS_ENABLED
-        std::cout << "Debug: DbTreeModel::refresh_from_database():  " << sql_query << std::endl;
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-      }
-      catch(const Glib::Exception& ex)
-      {
-        std::cout << "Debug: query string could not be converted to std::cout: " << ex.what() << std::endl;
-      }
-#endif //GLIBMM_EXCEPTIONS_ENABLED
+      m_data_model_rows_count = 0;
+      m_data_model_columns_count = m_columns_count;
+
+      std::cerr << "DbTreeModel::refresh_from_database(): error executing SQL." << std::endl;
+      ConnectionPool::handle_error_cerr_only();
+      return false; //No records were found.
     }
-
-    Glib::RefPtr<Gnome::Gda::SqlParser> parser = m_connection->get_gda_connection()->create_parser();
-    Glib::RefPtr<Gnome::Gda::Statement> stmt = parser->parse_string(sql_query);
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-    try
+    else
     {
-      //Specify the STATEMENT_MODEL_CURSOR, so that libgda only gets the rows that we actually use.
-      m_gda_datamodel = m_connection->get_gda_connection()->statement_execute_select(stmt, Gnome::Gda::STATEMENT_MODEL_CURSOR_FORWARD);
-
       // Use a DataAccessWrapper to allow random access. This is necessary
       // since we use move_to_row() on a created iterator in
       // fill_values_if_necessary(), which does not work if the iterator
@@ -526,51 +299,9 @@ bool DbTreeModel::refresh_from_database(const FoundSet& found_set)
       // TODO_Performance: The unnecessary (for PostgreSQL) extra indirection might theoretically make this slower.
       m_gda_datamodel = Gnome::Gda::DataAccessWrapper::create(m_gda_datamodel);
 
-      if(app && app->get_show_sql_debug())
-        std::cout << "  Debug: DbTreeModel::refresh_from_database(): The query execution has finished." << std::endl;
-
-      //Examine the columns in the returned DataModel:
-      /*
-      for(int col = 0; col < m_gda_datamodel->get_n_columns(); ++col)
-      {
-        Glib::RefPtr<Gnome::Gda::Column> column = m_gda_datamodel->describe_column(col);
-        std::cout << "  debug: column index=" << col << ", name=" << column->get_name() << ", type=" << g_type_name(column->get_g_type()) << std::endl;
-      }
-      */
-    }
-    catch(const Glib::Exception& ex)
-    {
-      std::cerr << "DbTreeModel::refresh_from_database(): Glib::Exception caught." << std::endl;
-      m_gda_datamodel.clear(); //So that it is 0, so we can handle it below.
-    }
-    catch(const std::exception& ex)
-    {
-      std::cerr << "DbTreeModel::refresh_from_database(): std::exception caught." << std::endl;
-      m_gda_datamodel.clear(); //So that it is 0, so we can handle it below.
-    }
-#else
-    std::auto_ptr<Glib::Error> error;
-    m_gda_datamodel = m_connection->get_gda_connection()->statement_execute_select(stmt, set, error);
-    if(error.get())
-    {
-      m_gda_datamodel.clear(); //So that it is 0, so we can handle it below.
-    }
-#endif //GLIBMM_EXCEPTIONS_ENABLED
-
-    if(!m_gda_datamodel)
-    {
-      m_data_model_rows_count = 0;
-      m_data_model_columns_count = m_columns_count;
-
-      std::cerr << "DbTreeModel::refresh_from_database(): error executing SQL: " << sql_query << std::endl;
-      ConnectionPool::handle_error();
-      return false; //No records were found.
-    }
-    else
-    {
       //This doesn't work with cursor-based models: const int count = m_gda_datamodel->get_n_rows();
       //because rows count is -1 until we have iterated to the last row.
-      const Glib::ustring sql_query_without_sort = Utils::build_sql_select_with_where_clause(m_found_set.m_table_name, m_column_fields, m_found_set.m_where_clause, m_found_set.m_extra_join, type_sort_clause(), m_found_set.m_extra_group_by);
+      Glib::RefPtr<Gnome::Gda::SqlBuilder> sql_query_without_sort = Utils::build_sql_select_with_where_clause(m_found_set.m_table_name, m_column_fields, m_found_set.m_where_clause, m_found_set.m_extra_join, type_sort_clause());
       const int count = Base_DB::count_rows_returned_by(sql_query_without_sort);
       if(count < 0)
       {
@@ -683,7 +414,7 @@ void DbTreeModel::get_value_vfunc(const TreeModel::iterator& iter, int column, G
             //const GType gtype_expected = column->get_g_type();
 
             result = row_details.get_value(const_cast<DbTreeModel&>(*this), column_sql, datamodel_row); //m_gda_datamodel->get_value_at(column_sql, datamodel_row); //dataRow.m_db_values[column];
-  
+
             /*
             if((result.get_value_type() != 0) && (result.get_value_type() != gtype_expected))
             {
@@ -700,7 +431,7 @@ void DbTreeModel::get_value_vfunc(const TreeModel::iterator& iter, int column, G
         GType debug_type = result.get_value_type();
         std::cout << "  debug: DbTreeModel::get_value_vfunc(): result value type: GType=" << debug_type << std::endl;
         if(debug_type)
-          std::cout << "    GType name=\"" << g_type_name(debug_type) << "\"" << std::endl; 
+          std::cout << "    GType name=\"" << g_type_name(debug_type) << "\"" << std::endl;
         */
 
         value_specific.set(result); //The compiler would complain if the type was wrong.
@@ -772,7 +503,7 @@ bool DbTreeModel::iter_nth_child_vfunc(const iterator& parent, int /* n */, iter
     return false;
   }
 
-  iter = iterator(); //Set is as invalid, as the TreeModel documentation says that it should be.  
+  iter = iterator(); //Set is as invalid, as the TreeModel documentation says that it should be.
   return false; //There are no children.
 }
 
@@ -866,7 +597,7 @@ bool DbTreeModel::create_iterator(const type_datamodel_row_index& datamodel_row,
   }
   else
   {
-    iter.set_stamp(m_stamp); 
+    iter.set_stamp(m_stamp);
     //Store the std::list iterator in the GtkTreeIter:
     //See also iter_next_vfunc()
 
@@ -891,7 +622,7 @@ bool DbTreeModel::get_iter_vfunc(const Path& path, iterator& iter) const
    if(sz > 1) //There are no children.
    {
      invalidate_iter(iter); //Set is as invalid, as the TreeModel documentation says that it should be.
-     return false; 
+     return false;
    }
 
    return iter_nth_root_child_vfunc(path[0], iter);
@@ -907,7 +638,7 @@ bool DbTreeModel::check_treeiter_validity(const iterator& iter) const
 {
   if(!(iter->get_model_gobject()))
     return false;
-    
+
   // Anything that modifies the model's structure should change the model's stamp,
   // so that old iters are ignored.
   return m_stamp == iter.get_stamp();
@@ -939,7 +670,7 @@ DbTreeModel::iterator DbTreeModel::append()
   ++m_count_extra_rows; //So that create_iterator() can succeed.
 
   //Create the row:
-  //datamodel_row->m_db_values.resize(m_columns_count); 
+  //datamodel_row->m_db_values.resize(m_columns_count);
 
   for(unsigned int column_number = 0; column_number < m_columns_count; ++column_number)
   {
@@ -1046,7 +777,7 @@ void DbTreeModel::set_key_value(const TreeModel::iterator& iter, const DbValue&
 }
 
 DbTreeModel::DbValue DbTreeModel::get_key_value(const TreeModel::iterator& iter) const
-{   
+{
   if(check_treeiter_validity(iter))
   {
     type_datamodel_row_index datamodel_row = get_datamodel_row_index_from_tree_row_iter(iter);
@@ -1174,22 +905,27 @@ void DbTreeModel::get_record_counts(gulong& total, gulong& found) const
     {
       //Ask the database how many records there are in the whole table:
       //TODO: Apparently, this is very slow:
-      const Glib::ustring sql_query = "SELECT count(*) FROM \"" + m_found_set.m_table_name + "\"";
-      Glib::RefPtr<Gnome::Gda::SqlParser> parser = m_connection->get_gda_connection()->create_parser();
-      Glib::RefPtr<Gnome::Gda::Statement> stmt = parser->parse_string(sql_query);
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-      Glib::RefPtr<Gnome::Gda::DataModel> datamodel = m_connection->get_gda_connection()->statement_execute_select(stmt);
-#else
-      std::auto_ptr<Glib::Error> error;
-      Glib::RefPtr<Gnome::Gda::DataModel> datamodel = m_connection->get_gda_connection()->statement_execute_select(stmt, error);
-      // Ignore error, datamodel presence is checked below
-#endif //GLIBMM_EXCEPTIONS_ENABLED
+
+      Glib::RefPtr<Gnome::Gda::SqlBuilder> builder =
+        Gnome::Gda::SqlBuilder::create(Gnome::Gda::SQL_STATEMENT_SELECT);
+
+      const guint id_function = builder->add_function("count", builder->add_id("*")); //TODO: Is * allowed here?
+      builder->add_field_id(id_function);
+
+      builder->select_add_target(m_found_set.m_table_name);
+
+      Glib::RefPtr<Gnome::Gda::DataModel> datamodel = DbUtils::query_execute_select(builder);
 
       if(datamodel)
       {
         if(datamodel->get_n_rows())
         {
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
           Gnome::Gda::Value value = datamodel->get_value_at(0, 0);
+#else
+          std::auto_ptr<Glib::Error> value_error;
+          Gnome::Gda::Value value = datamodel->get_value_at(0, 0, value_error);
+#endif
           total = (gulong)value.get_int64(); //I discovered that it's a int64 by trying it.
         }
       }
